/* pcl725.c - Application to use the Advantech PCL-725 and ADClone ACL-725
	      board (via the corresponding device driver) */
 
/* Written 1996 by Werner Almesberger, EPFL LRC */
 

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include "pcl725.h"


#define DEFAULT_DEV_NAME "/dev/" DEV_NAME "0"


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-d dev] op [arg]\n\n",name);
    fprintf(stderr,"operations: in, status, set <bitmask>, on <line|mask>,\n");
    fprintf(stderr,"            off <line|mask>, toggle <line|mask>\n");
    exit(1);
}


int main(int argc,char **argv)
{
    const char *dev,*name;
    char *end;
    int cmd,arg,fd,result;

    dev = DEFAULT_DEV_NAME;
    name = argv[0];
    if (argc < 2) usage(name);
    if (!strcmp(argv[1],"-d")) {
	if (argc < 4) usage(name);
	dev = argv[2];
	argc -= 2;
	argv += 2;
    }
    if (!strcmp(argv[1],"in")) cmd = PCL725_INP_GET;
    else if (!strcmp(argv[1],"status")) cmd = PCL725_REL_GET;
    else if (!strcmp(argv[1],"set")) cmd = PCL725_REL_SET;
    else if (!strcmp(argv[1],"on")) cmd = PCL725_REL_ON;
    else if (!strcmp(argv[1],"off")) cmd = PCL725_REL_OFF;
    else if (!strcmp(argv[1],"toggle")) cmd = PCL725_REL_TOGGLE;
    else usage(name);
    switch (cmd) {
	case PCL725_INP_GET:
	case PCL725_REL_GET:
	    if (argc > 2) usage(name);
	    break;
	case PCL725_REL_ON:
	case PCL725_REL_OFF:
	case PCL725_REL_TOGGLE:
	    if (argv[2][0] != '0' || argv[2][1] != 'x') {
		arg = strtoul(argv[2],&end,10);
		if (*end) usage(name);
		if (arg > 7) usage(name);
		arg = 1 << arg;
		break;
	    }
	    /* fall through */
	case PCL725_REL_SET:
	    arg = strtoul(argv[2],&end,16);
	    if (*end) usage(name);
	    if (arg & ~0xff) usage(name);
	    break;
	default:
	    /* never reached */
    }
    if ((fd = open(dev,O_RDONLY)) < 0) {
	perror(dev);
	return 1;
    }
    result = ioctl(fd,cmd,arg);
    if (result < 0) {
	perror("ioctl");
	return 1;
    }
    if (cmd == PCL725_INP_GET || cmd == PCL725_REL_GET)
	printf("0x%x\n",result);
    return 0;
}
